package com.info666.infraredRemote.controller;

import com.aliyuncs.dysmsapi.model.v20170525.SendSmsResponse;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.core.sms.SendMessage;
import com.ruoyi.common.utils.RedisKeys;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.ip.IpUtils;
import com.ruoyi.system.domain.UserInfo;
import com.ruoyi.system.domain.vo.SmsResponseVo;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.concurrent.TimeUnit;
import java.util.regex.Pattern;

@RestController
@RequestMapping("/api/v1/sms")
public class SmsController {

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private SendMessage sendMessage;


    @PostMapping("/send")
    public R<SmsResponseVo> sendSms(@RequestBody UserInfo param, HttpServletRequest request) {
        if (StringUtils.isEmpty(param.getUserPhone()) || Pattern.matches("^1[3-9]\\d{9}$\n", param.getUserPhone())) {
            return R.fail("手机号格式不正确");
        }

        //每个公网id地址1天只能发100条
        //每个手机号一天只能发10条,一分钟只能发一条
        String ip = IpUtils.getIpAddr(request);
        DateTimeFormatter dtf = DateTimeFormatter.ofPattern("yyyyMMdd");
        String strDate = dtf.format(LocalDate.now());
        String ipKey = RedisKeys.INCR_IP + strDate + "_" + ip;
        String phoneIconKey = RedisKeys.INCR_PHONE + strDate + "_" + param.getUserPhone();
        String phone = RedisKeys.SMS_PHONE_TIME + param.getUserPhone();
        //一分钟发送一次
        if (redisTemplate.hasKey(phone)) {
            //存在
            return R.fail("1分钟内只能发送一次,请稍后再试!");
        } else {
            //不存在
            redisTemplate.opsForValue().set(phone, 1, 1L, TimeUnit.MINUTES);
        }

        //手机号
        if (redisTemplate.hasKey(phoneIconKey)) {
            //存在
            Integer increment = redisCache.getCacheObject(phoneIconKey);
            if (increment > 10) {
                return R.fail("该手机号发送次数过多,已禁止发送!");
            }
        } else {
            //不存在
            redisTemplate.opsForValue().set(phoneIconKey, 1, 1L, TimeUnit.DAYS);
        }

        //ip
        if (redisTemplate.hasKey(ipKey)) {
            //存在
            Integer increment = redisCache.getCacheObject(ipKey);
            if (increment > 100) {
                return R.fail("发送次数过多,已禁止发送!");
            }
        } else {
            //不存在
            redisTemplate.opsForValue().set(ipKey, 1, 1L, TimeUnit.DAYS);
        }

        // 通过RandomStringUtils生成随机验证码
        String code = RandomStringUtils.randomNumeric(6);
        SmsResponseVo smsResponse = new SmsResponseVo();
        try {
            //手机号和验证码进行发短信
            SendSmsResponse sendSmsResponse = sendMessage.sendSms(param.getUserPhone(), code);

            if (sendSmsResponse.getMessage().equals("OK")) {
                redisCache.setCacheObject(RedisKeys.SMS_PHONE_CODE + param.getUserPhone(), code, 5, TimeUnit.MINUTES);
                smsResponse.setCode(sendSmsResponse.getCode());
                smsResponse.setMessage(sendSmsResponse.getMessage());
                smsResponse.setKeyCode(RedisKeys.SMS_PHONE_CODE + param.getUserPhone());

            } else {
                return R.fail(sendSmsResponse.getMessage());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return R.ok(smsResponse);
    }
}
